@@ -157,6 +157,18 @@ class Compose(ODCSBase):
     pungi_compose_id = db.Column(db.String, nullable=True)
     # Full Pungi configuration dump, used only for raw_config source type.
     pungi_config_dump = db.Column(db.String, nullable=True)
+    # UUID of the celery task.
+    celery_task_id = db.Column(db.String, nullable=True)
+    # Target directory in which the compose is stored. This is `conf.target_dir`
+    # by default.
+    target_dir = db.Column(db.String, nullable=True)
+
+    @property
+    def on_default_target_dir(self):
+        """
+        True if this compose is stored on default `conf.target_dir`.
+        """
+        return self.target_dir is None or self.target_dir == conf.target_dir
 
     @classmethod
     def create(cls, session, owner, source_type, source, results,
@@ -164,7 +176,7 @@ class Compose(ODCSBase):
                koji_event=None, arches=None, multilib_arches=None,
                multilib_method=None, builds=None, lookaside_repos=None,
                modular_koji_tags=None, module_defaults_url=None,
-               label=None, compose_type=None):
+               label=None, compose_type=None, target_dir=None):
         now = datetime.utcnow()
         compose = cls(
             owner=owner,
@@ -187,6 +199,7 @@ class Compose(ODCSBase):
             module_defaults_url=module_defaults_url,
             label=label,
             compose_type=compose_type,
+            target_dir=target_dir or conf.target_dir,
         )
         session.add(compose)
         return compose
@@ -227,6 +240,9 @@ class Compose(ODCSBase):
             # Set pungi_compose_id to None, because it is regenerated once
             # this copied Compose is started.
             pungi_compose_id=None,
+            # Also reset celery task_id
+            celery_task_id=None,
+            target_dir=compose.target_dir,
         )
         session.add(compose)
         return compose
@@ -250,7 +266,7 @@ class Compose(ODCSBase):
         # a race between we start Pungi and when Pungi generates that dir,
         # so just use `glob` to find out the rigth directory.
         glob_str = os.path.join(
-            conf.target_dir, "odcs-%d-1-*.n.0" % self.id)
+            self.target_dir, "odcs-%d-1-*.n.0" % self.id)
         toplevel_dirs = glob.glob(glob_str)
         if toplevel_dirs:
             return toplevel_dirs[0]
@@ -262,7 +278,7 @@ class Compose(ODCSBase):
             toplevel_dir = self.toplevel_work_dir
             if toplevel_dir:
                 return toplevel_dir
-        return os.path.join(conf.target_dir, self.latest_dir)
+        return os.path.join(self.target_dir, self.latest_dir)
 
     @property
     def result_repo_dir(self):
@@ -277,6 +293,9 @@ class Compose(ODCSBase):
         """
         Returns public URL to compose directory with per-arch repositories.
         """
+        if not self.on_default_target_dir:
+            return ""
+
         target_dir_url = conf.target_dir_url
 
         return target_dir_url + "/" \
@@ -295,6 +314,9 @@ class Compose(ODCSBase):
         """
         Returns public URL to repofile.
         """
+        if not self.on_default_target_dir:
+            return ""
+
         target_dir_url = conf.target_dir_url
         return target_dir_url + "/" \
             + os.path.join(self.latest_dir, "compose", "Temporary",
@@ -323,6 +345,12 @@ class Compose(ODCSBase):
             if self.results & value:
                 results.append(name)
 
+        if self.on_default_target_dir:
+            target_dir = "default"
+        else:
+            inverse_extra_target_dirs = {v: k for k, v in conf.extra_target_dirs.items()}
+            target_dir = inverse_extra_target_dirs.get(self.target_dir, "unknown")
+
         ret = {
             'id': self.id,
             'owner': self.owner,
@@ -355,6 +383,7 @@ class Compose(ODCSBase):
             'label': self.label,
             'compose_type': self.compose_type,
             'pungi_compose_id': self.pungi_compose_id,
+            'target_dir': target_dir,
         }
 
         if full:
