@@ -161,7 +161,6 @@ class PungiConfig(BasePungiConfig):
         self.koji_profile = conf.koji_profile
         self.pkgset_koji_inherit = True
         self.lookaside_repos = lookaside_repos.split(" ") if lookaside_repos else []
-        self.include_devel_modules = []
         if arches:
             self.arches = arches
         else:
@@ -201,8 +200,6 @@ class PungiConfig(BasePungiConfig):
             self.gather_method = "nodeps"
             self.module_defaults_url = module_defaults_url.split(" ") if module_defaults_url else []
 
-            self._sort_out_devel_modules()
-
             if self.packages:
                 raise ValueError("Exact packages cannot be set for MODULE "
                                  "source type.")
@@ -217,31 +214,6 @@ class PungiConfig(BasePungiConfig):
 
         self.check_deps = bool(flags & COMPOSE_FLAGS["check_deps"])
 
-    def _sort_out_devel_modules(self):
-        """
-        Helper method filtering out "-devel" modules from `self.source`
-        and adding them to `include_devel_modules` list.
-        """
-        source_list = self.source.split(" ")
-        new_source = []
-        for nsvc in source_list:
-            n, s, v, c = nsvc.split(":")
-
-            # It does not have -devel suffix, so it is not -devel module.
-            if not n.endswith("-devel"):
-                new_source.append(nsvc)
-                continue
-
-            # If it is -devel module, there must exist the non-devel
-            # counterpart.
-            non_devel_nsvc = ":".join([n[:-len("-devel")], s, v, c])
-            if non_devel_nsvc not in source_list:
-                new_source.append(nsvc)
-                continue
-
-            self.include_devel_modules.append(":".join([n, s]))
-        self.source = " ".join(new_source)
-
     @property
     def source_type_str(self):
         return INVERSE_PUNGI_SOURCE_TYPE_NAMES[self.source_type]
@@ -431,9 +403,9 @@ class Pungi(object):
         try:
             td = tempfile.mkdtemp()
             self._write_cfgs(td)
-            compose_dir = self._prepare_compose_dir(compose, td, conf.target_dir)
+            compose_dir = self._prepare_compose_dir(compose, td, compose.target_dir)
             self.pungi_cfg.validate(td, compose_dir)
-            pungi_cmd = self.get_pungi_cmd(td, conf.target_dir, compose_dir)
+            pungi_cmd = self.get_pungi_cmd(td, compose.target_dir, compose_dir)
 
             # Commit the session to ensure that all the `compose` changes are
             # stored database before executing the compose and are not just
@@ -539,8 +511,9 @@ class PungiLogs(object):
                 continue
             errors += error
 
-        errors = errors.replace(
-            conf.target_dir, conf.target_dir_url)
+        if self.compose.on_default_target_dir:
+            errors = errors.replace(
+                conf.target_dir, conf.target_dir_url)
         return errors
 
     def get_config_dump(self):
