@@ -29,36 +29,44 @@ from six.moves.urllib.parse import urlparse, parse_qs
 from odcs.server import conf
 
 import gi
-gi.require_version('Modulemd', '1.0') # noqa
+gi.require_version('Modulemd', '2.0') # noqa
 from gi.repository import Modulemd
 
 
-def make_module(name, stream, version, requires={}, mdversion=1):
-    mmd = Modulemd.Module()
-    mmd.set_mdversion(mdversion)
-    mmd.set_name(name)
-    mmd.set_stream(stream)
+def dump_mmd(mmd):
+    mod_index = Modulemd.ModuleIndex.new()
+    mod_index.add_module_stream(mmd)
+    return mod_index.dump_to_string()
+
+
+def make_module(name, stream, version, requires={}, mdversion=1,
+                context=None, state=5):
+    if mdversion == 1:
+        mmd = Modulemd.ModuleStreamV1.new(name, stream)
+    else:
+        mmd = Modulemd.ModuleStreamV2.new(name, stream)
     mmd.set_version(version)
+    mmd.set_context(context or '00000000')
     mmd.set_summary("foo")
     mmd.set_description("foo")
-    licenses = Modulemd.SimpleSet()
-    licenses.add("GPL")
-    mmd.set_module_licenses(licenses)
+    mmd.add_module_license("GPL")
 
     if mdversion == 1:
-        mmd.set_requires(requires)
+        for req_name, req_stream in requires.items():
+            mmd.add_runtime_requirement(req_name, req_stream)
     else:
         deps = Modulemd.Dependencies()
         for req_name, req_stream in requires.items():
-            deps.add_requires_single(req_name, req_stream)
-        mmd.set_dependencies((deps, ))
+            deps.add_runtime_stream(req_name, req_stream)
+        mmd.add_dependencies(deps)
 
     return {
         'name': name,
         'stream': stream,
         'version': str(version),
-        'context': '00000000',
-        'modulemd': mmd.dumps()
+        'context': context or '00000000',
+        'modulemd': dump_mmd(mmd),
+        'state': state,
     }
 
 
@@ -102,22 +110,35 @@ TEST_MBS_MODULES_MMDv2 = [
 
     make_module('moduleD', 'f26', 20170806000000, {}, 2),
 
+    # module builds in "done" state.
+    make_module('testmodule', 'master', 20180515074419, {}, 2, state=3),
+
     # test_composerthread.py
     make_module('testmodule', 'master', 20170515074418, {}, 2),
-    make_module('testmodule', 'master', 20170515074419, {}, 2)
+    make_module('testmodule', 'master', 20170515074419, {}, 2),
+
+    # multiple contexts
+    make_module('parent', 'master', 1, {}, 2, context="a"),
+    make_module('parent', 'master', 1, {}, 2, context="b"),
+    make_module('testcontexts', 'master', 1, {"parent": "master"},
+                2, context="a"),
+    make_module('testcontexts', 'master', 1, {"parent": "master"},
+                2, context="b"),
 ]
 
 
 def mock_mbs(mdversion=2):
     """
-    Decorator that sets up a test environment so that calls to the PDC to look up
-    modules are redirected to return results from the TEST_MODULES array above.
+    Decorator that sets up a test environment so that calls to the MBS to look
+    up modules are redirected to return results from the TEST_MODULES array
+    above.
     """
     def wrapper(f):
         @wraps(f)
         def wrapped(*args, **kwargs):
             def handle_module_builds(request):
                 query = parse_qs(urlparse(request.url).query)
+                states = [int(s) for s in query['state']]
                 nsvc = query['nsvc'][0]
                 nsvc_parts = nsvc.split(":")
                 nsvc_keys = ["name", "stream", "version", "context"]
@@ -137,6 +158,8 @@ def mock_mbs(mdversion=2):
                         if key in nsvc_dict and nsvc_dict[key] != module[key]:
                             skip = True
                             break
+                    if module["state"] not in states:
+                        skip = True
                     if skip:
                         continue
                     body["items"].append(module)
