@@ -22,11 +22,12 @@
 
 import os
 import time
+from datetime import datetime, timedelta
 
-from mock import patch, MagicMock
+from mock import patch, MagicMock, call
 
 import odcs.server
-from odcs.server import db, app
+from odcs.server import db, app, conf
 from odcs.server.models import Compose
 from odcs.common.types import COMPOSE_STATES, COMPOSE_RESULTS, COMPOSE_FLAGS
 from odcs.server.backend import ComposerThread, resolve_compose
@@ -354,3 +355,119 @@ class TestComposerThreadLostComposes(ModelsBaseTest):
         self.composer.currently_generating += [done.id, generating.id]
         self.composer.do_work()
         self.assertEqual(self.composer.currently_generating, [generating.id])
+
+
+class TestComposerThreadStuckWaitComposes(ModelsBaseTest):
+    maxDiff = None
+
+    def setUp(self):
+        self.client = app.test_client()
+        super(TestComposerThreadStuckWaitComposes, self).setUp()
+        self.composer = ComposerThread()
+
+        self.patch_generate_new_compose = patch(
+            "odcs.server.backend.ComposerThread.generate_new_compose")
+        self.generate_new_compose = self.patch_generate_new_compose.start()
+
+    def tearDown(self):
+        super(TestComposerThreadStuckWaitComposes, self).tearDown()
+        self.patch_generate_new_compose.stop()
+
+    def _add_test_compose(self, state, time_submitted=None,
+                          time_started=None,
+                          source_type=PungiSourceType.KOJI_TAG):
+        compose = Compose.create(
+            db.session, "unknown", source_type, "f26",
+            COMPOSE_RESULTS["repository"], 60, "", 0)
+        compose.state = state
+        if time_submitted:
+            compose.time_submitted = time_submitted
+        if time_started:
+            compose.time_started = time_started
+        db.session.add(compose)
+        db.session.commit()
+        return compose
+
+    def test_pickup_waiting_composes_generating_state(self):
+        time_submitted = datetime.utcnow() - timedelta(minutes=5)
+        composes = []
+        for i in range(10):
+            composes.append(self._add_test_compose(
+                COMPOSE_STATES["wait"], time_submitted=time_submitted))
+        composes = sorted(composes, key=lambda c: c.id)
+        self.composer.pickup_waiting_composes()
+        self.generate_new_compose.assert_has_calls([
+            call(composes[0]), call(composes[1]), call(composes[2]),
+            call(composes[3])])
+
+    def test_pickup_waiting_composes_generating_state_not_old_enough(self):
+        composes = []
+        for i in range(10):
+            composes.append(self._add_test_compose(
+                COMPOSE_STATES["wait"]))
+        composes = sorted(composes, key=lambda c: c.id)
+        self.composer.pickup_waiting_composes()
+        self.generate_new_compose.assert_not_called()
+
+    def test_pickup_waiting_composes_generating_state_old(self):
+        time_submitted = datetime.utcnow() - timedelta(days=5)
+        composes = []
+        for i in range(10):
+            composes.append(self._add_test_compose(
+                COMPOSE_STATES["wait"], time_submitted=time_submitted))
+        composes = sorted(composes, key=lambda c: c.id)
+        self.composer.pickup_waiting_composes()
+        self.generate_new_compose.assert_not_called()
+
+    def test_generate_lost_composes_generating_state(self):
+        composes = []
+        for i in range(10):
+            composes.append(self._add_test_compose(COMPOSE_STATES["generating"]))
+        composes = sorted(composes, key=lambda c: c.id)
+        self.composer.pickup_waiting_composes()
+        self.generate_new_compose.assert_not_called()
+
+    def test_pickup_waiting_composes_no_limit_for_pulp(self):
+        time_submitted = datetime.utcnow() - timedelta(minutes=5)
+        composes = []
+        for i in range(10):
+            composes.append(self._add_test_compose(
+                COMPOSE_STATES["wait"], time_submitted=time_submitted))
+        for i in range(10):
+            composes.append(self._add_test_compose(
+                COMPOSE_STATES["wait"], time_submitted=time_submitted,
+                source_type=PungiSourceType.PULP))
+        composes = sorted(composes, key=lambda c: c.id)
+        self.composer.pickup_waiting_composes()
+        self.generate_new_compose.assert_has_calls([
+            call(composes[0]), call(composes[1]), call(composes[2]),
+            call(composes[3]), call(composes[10]), call(composes[11]),
+            call(composes[12]), call(composes[13]), call(composes[14]),
+            call(composes[15]), call(composes[16]), call(composes[17]),
+            call(composes[18]), call(composes[19])])
+
+    def test_fail_lost_generating_composes(self):
+        t = datetime.utcnow() - timedelta(seconds=2 * conf.pungi_timeout)
+
+        time_submitted = t - timedelta(minutes=6)
+        time_started = t - timedelta(minutes=5)
+        compose_to_fail = self._add_test_compose(
+            COMPOSE_STATES["generating"],
+            time_submitted=time_submitted,
+            time_started=time_started,
+        )
+
+        time_submitted = t + timedelta(minutes=4)
+        time_started = t + timedelta(minutes=5)
+        compose_to_keep = self._add_test_compose(
+            COMPOSE_STATES["generating"],
+            time_submitted=time_submitted,
+            time_started=time_started,
+        )
+
+        self.composer.fail_lost_generating_composes()
+        db.session.commit()
+        db.session.expire_all()
+
+        self.assertEqual(compose_to_fail.state, COMPOSE_STATES["failed"])
+        self.assertEqual(compose_to_keep.state, COMPOSE_STATES["generating"])
