@@ -79,7 +79,8 @@ class TestModels(ModelsBaseTest):
                          'label': None,
                          'compose_type': None,
                          'pungi_compose_id': None,
-                         'pungi_config_dump': 'test'}
+                         'pungi_config_dump': 'test',
+                         'target_dir': 'default'}
         self.assertEqual(c.json(True), expected_json)
 
     def test_create_copy(self):
@@ -117,7 +118,7 @@ class TestModels(ModelsBaseTest):
             if c.name in ["id", "state", "state_reason", "time_to_expire",
                           "time_done", "time_submitted", "time_removed",
                           "removed_by", "reused_id", "koji_task_id",
-                          "time_started", "pungi_compose_id"]:
+                          "time_started", "pungi_compose_id", "celery_task_id"]:
                 assertMethod = self.assertNotEqual
             else:
                 assertMethod = self.assertEqual
