@@ -30,7 +30,7 @@ import flask
 
 from werkzeug.exceptions import BadRequest
 from freezegun import freeze_time
-from mock import patch, PropertyMock
+from mock import patch, PropertyMock, call
 
 import odcs.server.auth
 
@@ -119,7 +119,8 @@ class ViewBaseTest(ModelsBaseTest):
                     'compose_types': ["test", "nightly"]
                 },
                 'dev3': {
-                    'source_types': ['tag']
+                    'source_types': ['tag'],
+                    'target_dirs': ["releng-private"]
                 }
             }
         }
@@ -318,7 +319,8 @@ class TestViews(ViewBaseTest):
                          'module_defaults_url': None,
                          'label': None,
                          'compose_type': 'test',
-                         'pungi_compose_id': None}
+                         'pungi_compose_id': None,
+                         'target_dir': 'default'}
         self.assertEqual(data, expected_json)
 
         db.session.expire_all()
@@ -534,6 +536,61 @@ class TestViews(ViewBaseTest):
         c = db.session.query(Compose).filter(Compose.id == 1).one()
         self.assertEqual(c.state, COMPOSE_STATES["wait"])
 
+    def test_submit_build_target_dir_unknown(self):
+        with self.test_request_context(user='dev'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'f26'},
+                 'target_dir': 'foo'}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(data['status'], 400)
+        self.assertEqual(data['error'], 'Bad Request')
+        self.assertEqual(data['message'], 'Unknown "target_dir" "foo"')
+
+    @patch.object(odcs.server.config.Config, 'extra_target_dirs',
+                  new={"releng-private": "/tmp/private"})
+    def test_submit_build_target_not_allowed(self):
+        with self.test_request_context(user='dev'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'f26'},
+                 'target_dir': 'releng-private'}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(data['status'], 403)
+        self.assertEqual(data['error'], 'Forbidden')
+        self.assertEqual(
+            data['message'],
+            'User dev not allowed to operate with compose with target_dirs=releng-private.')
+
+    @patch.object(odcs.server.config.Config, 'extra_target_dirs',
+                  new={"releng-private": "/tmp/private"})
+    def test_submit_build_target_dir(self):
+        with self.test_request_context(user='dev3'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'f26'},
+                 'target_dir': 'releng-private'}))
+            self.assertEqual(rv.status, '200 OK')
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(data['target_dir'], 'releng-private')
+
+        db.session.expire_all()
+        c = db.session.query(Compose).filter(Compose.id == 3).one()
+        self.assertEqual(c.state, COMPOSE_STATES["wait"])
+        self.assertEqual(c.target_dir, "/tmp/private")
+
     def test_submit_build_module_defaults_url(self):
         with self.test_request_context(user='dev'):
             flask.g.oidc_scopes = [
@@ -913,7 +970,7 @@ class TestViews(ViewBaseTest):
 
     def test_delete_not_allowed_states_compose(self):
         for state in COMPOSE_STATES.keys():
-            if state not in ['done', 'failed']:
+            if state not in ['wait', 'done', 'failed']:
                 new_c = Compose.create(
                     db.session, "unknown", PungiSourceType.MODULE, "testmodule:master",
                     COMPOSE_RESULTS["repository"], 60)
@@ -951,6 +1008,9 @@ class TestViews(ViewBaseTest):
         self.assertEqual(data['error'], 'Not Found')
 
     def test_delete_compose_with_non_admin_user(self):
+        self.c1.state = COMPOSE_STATES["failed"]
+        db.session.commit()
+
         with self.test_request_context(user='dev'):
             flask.g.oidc_scopes = [
                 '{0}{1}'.format(conf.oidc_base_namespace, 'delete-compose')
@@ -1165,7 +1225,8 @@ class TestViews(ViewBaseTest):
                          'module_defaults_url': None,
                          'label': None,
                          'compose_type': 'test',
-                         'pungi_compose_id': None}
+                         'pungi_compose_id': None,
+                         'target_dir': 'default'}
         self.assertEqual(data, expected_json)
 
         db.session.expire_all()
@@ -1441,3 +1502,58 @@ class TestViewsRawConfig(ViewBaseTest):
         self.assertEqual(c.source, 'pungi_cfg#hash')
         self.assertEqual(c.label, 'Beta-1.2')
         self.assertEqual(c.compose_type, 'nightly')
+
+
+class TestViewsCancelCompose(ViewBaseTest):
+    maxDiff = None
+
+    def setup_test_data(self):
+        self.initial_datetime = datetime(
+            year=2016, month=1, day=1, hour=0, minute=0, second=0
+        )
+        with freeze_time(self.initial_datetime):
+            self.c1 = Compose.create(
+                db.session, "dev2", PungiSourceType.MODULE, "testmodule:master",
+                COMPOSE_RESULTS["repository"], 60)
+            db.session.commit()
+            self.task_id = "71267f28-5194-4720-b57b-a665fabdb012"
+            self.c1.celery_task_id = self.task_id
+            db.session.commit()
+            self.c1_id = self.c1.id
+
+    @patch("odcs.server.views.CELERY_AVAILABLE", new=False)
+    def test_no_celery(self):
+        with self.test_request_context(user='dev2'):
+            resp = self.client.delete("/api/1/composes/%s" % self.c1.id)
+        # Without celery we can't cancel, so the code should try to delete the
+        # compose and fail on user not being an admin.
+        self.assertEqual(resp.status_code, 403)
+        data = json.loads(resp.get_data(as_text=True))
+        self.assertEqual(data["status"], 403)
+        self.assertEqual(data["message"], "User dev2 is not in role admins.")
+
+    def test_bad_owner(self):
+        with self.test_request_context(user='dev1'):
+            resp = self.client.delete("/api/1/composes/%s" % self.c1.id)
+        self.assertEqual(resp.status_code, 403)
+        data = json.loads(resp.get_data(as_text=True))
+        self.assertEqual(data["status"], 403)
+        self.assertEqual(
+            data["message"],
+            "Compose (id=%s) can not be canceled, it's owned by someone else.",
+        )
+
+    @patch("odcs.server.views.celery_app")
+    def test_cancel(self, app):
+        with self.test_request_context(user='dev2'):
+            resp = self.client.delete("/api/1/composes/%s" % self.c1_id)
+        self.assertEqual(resp.status_code, 202)
+        self.assertEqual(app.mock_calls, [call.control.revoke(self.task_id)])
+        data = json.loads(resp.get_data(as_text=True))
+        self.assertEqual(data["status"], 202)
+        self.assertEqual(
+            data["message"], "Compose (id=%s) has been canceled" % self.c1_id
+        )
+        c1 = db.session.query(Compose).filter(Compose.id == self.c1_id).first()
+        self.assertEqual(c1.state, COMPOSE_STATES["failed"])
+        self.assertEqual(c1.state_reason, "Canceled by dev2")
